from django.contrib.auth.decorators import login_required
from django.contrib.admin.views.decorators import staff_member_required
from django.db.models import Q
from django.http import HttpResponseRedirect
from django.views.generic.base import TemplateView, RedirectView
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView, FormView, UpdateView
from django.views.generic.list import ListView
from django.core.urlresolvers import reverse_lazy
from django.core.exceptions import PermissionDenied


from .models import User
from .forms import MemberEditForm, MemberCreateForm, MemberEmissionForm


class ProfileView(TemplateView):
    template_name = 'aa/profile.html'


profile_view = login_required(ProfileView.as_view())


class ProfileContactEditView(UpdateView):
    model = User
    fields = ['phone', 'mobile', 'share_contact_details']
    success_url = reverse_lazy('profile-view')
    template_name = 'aa/profile_form.html'

    def get_object(self):
        return self.request.user


profile_contact_edit = login_required(ProfileContactEditView.as_view())


class MembersListView(ListView):
    paginate_by = 10

    def get_queryset(self):
        qs = User.objects.all()
        if not (self.request.user.is_staff and self.request.GET.get('q')):
            # filter on active users unless a search term is given
            qs = qs.filter(is_active=True)
        if self.request.GET.get('q'):
            for part in self.request.GET.get('q').split():
                part = part.strip()
                if not part:
                    continue
                if part.lower() in ('ca', 'cp'):
                    qs = qs.filter(groups__name=part.upper())
                    continue
                qs = qs.filter(
                    Q(first_name__icontains=part)
                    | Q(last_name__icontains=part)
                    | Q(emissions__title__icontains=part)
                    | Q(emissions__slug__icontains=part)
                )
        qs = qs.distinct()
        return qs


members_list_view = login_required(MembersListView.as_view())


class MemberView(DetailView):
    model = User


member_view = login_required(MemberView.as_view())


class MemberEditView(UpdateView):
    model = User
    form_class = MemberEditForm

    def get_object(self):
        if not self.request.user.is_staff:
            raise PermissionDenied()
        return super().get_object()

    def get_success_url(self):
        return reverse_lazy('member-view', kwargs={'pk': self.get_object().id})


member_edit = login_required(MemberEditView.as_view())


class MemberCreateView(CreateView):
    model = User
    form_class = MemberCreateForm

    def get_object(self):
        if not self.request.user.is_staff:
            raise PermissionDenied()
        return super().get_object()

    def form_valid(self, form):
        response = super().form_valid(form)
        self.object.username = self.object.email
        self.object.set_password(form.cleaned_data['password'])
        self.object.save()
        return response

    def get_success_url(self):
        return reverse_lazy('member-view', kwargs={'pk': self.object.id})


member_new = login_required(MemberCreateView.as_view())


class MemberEmissionsView(FormView):
    form_class = MemberEmissionForm
    template_name = 'aa/member_emissions.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['member'] = User.objects.get(id=self.kwargs['pk'])
        return context

    def form_valid(self, form):
        response = super().form_valid(form)
        member = User.objects.get(id=self.kwargs['pk'])
        member.emissions.add(form.cleaned_data['emission'])
        return response

    def get_success_url(self):
        return reverse_lazy('member-view', kwargs={'pk': self.kwargs['pk']})


member_emissions = login_required(MemberEmissionsView.as_view())


class MemberEmissionRemoveView(RedirectView):
    def get_redirect_url(self, *args, **kwargs):
        if not self.request.user.is_staff:
            raise PermissionDenied()
        member = User.objects.get(id=kwargs['pk'])
        member.emissions.remove(kwargs['em_pk'])
        return reverse_lazy('member-view', kwargs={'pk': member.id})


member_emission_remove = login_required(MemberEmissionRemoveView.as_view())



@staff_member_required
def mark_as_active(request, pk):
    User.objects.filter(pk=pk).update(is_active=True)
    return HttpResponseRedirect(reverse_lazy('member-view', kwargs={'pk': pk}))


@staff_member_required
def mark_as_inactive(request, pk):
    User.objects.filter(pk=pk).update(is_active=False)
    return HttpResponseRedirect(reverse_lazy('member-view', kwargs={'pk': pk}))
