import datetime

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.forms import AuthenticationForm
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import HttpResponseNotAllowed, HttpResponseRedirect, JsonResponse
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import TemplateView

from panikdb.context_processors import internal_ip

from emissions.models import Nonstop
from nonstop.models import Track, SomaLogLine

from .models import PigeEmailRequest


def is_internal(request):
    return internal_ip(request).get('internal_ip')


class RegieHome(TemplateView):
    template_name = 'regie-home.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['login_form'] = AuthenticationForm()
        if settings.PIGE_DOWNLOAD_BASE_URL:
            context['has_pige'] = True
            context['pige_minimum_date'] = datetime.datetime.now() - datetime.timedelta(days=100)
            context['pige_hours'] = ['%02d' % x for x in range(24)]
            context['pige_minutes'] = ['%02d' % x for x in range(0, 60, 15)]
        return context


regie_home = RegieHome.as_view()


def pige_post(request, *args, **kwargs):
    if request.method != 'POST':
        return HttpResponseNotAllowed(['POST'])
    # studio2-20160515-13h45-14h00m05.flac
    name = '%(src)s-%(date)s-%(start_hour)sh%(start_min)s-%(end_hour)sh%(end_min)s' % {
            'src': request.POST['src'],
            'date': request.POST['date'].replace('-', ''),
            'start_hour': request.POST['start_hour'],
            'start_min': request.POST['start_min'],
            'end_hour': request.POST['end_hour'],
            'end_min': request.POST['end_min']}
    if is_internal(request):
        name = name + '.wav'
    else:
        name = name + '.ogg'
    if 'sendmail' in request.POST:
        email = request.POST['email']
        PigeEmailRequest.objects.create(email=email, download_spec=name)
        messages.info(request, _('An email with a download link will soon be sent to %s.') % email)
        return HttpResponseRedirect(reverse('regie-home'))
    else:  # direct download
        return HttpResponseRedirect(settings.PIGE_DOWNLOAD_BASE_URL + '/' + name)


def json_tracks(tracks):
    return JsonResponse({'data': [
        {
            'id': track.id,
            'title': track.title,
            'artist': track.artist.name if track.artist_id else None,
            'instru': track.instru,
            'language': track.language,
            'duration': '%d:%02d' % (
                track.duration.total_seconds() / 60,
                track.duration.total_seconds() % 60),
            } for track in tracks]})


def regie_tracks(request):
    now = datetime.datetime.now()
    current_zone = None
    for nonstop in Nonstop.objects.all():
        if (nonstop.start < nonstop.end and (
                now.time() >= nonstop.start and now.time() < nonstop.end)) or \
           (nonstop.start > nonstop.end and (
                now.time() >= nonstop.start or now.time() < nonstop.end)):
            current_zone = nonstop
            break

    tracks = Track.objects.filter(duration__isnull=False)
    tracks = tracks.filter(duration__gte=datetime.timedelta(minutes=2, seconds=30))
    tracks = tracks.filter(duration__lt=datetime.timedelta(minutes=4, seconds=30))
    if current_zone:
        tracks = tracks.filter(nonstop_zones__in=[current_zone.id])
    tracks = tracks.exclude(language__isnull=True).exclude(language='')
    tracks = tracks.order_by('?')
    return json_tracks(tracks[:5])


def regie_tracks_search(request):
    q = request.GET['q']
    tracks = Track.objects.filter(
            duration__isnull=False).filter(
            Q(title__icontains=q.lower()) | Q(artist__name__icontains=q.lower()))
    tracks = tracks.order_by('?')
    return json_tracks(tracks[:5])


def playing(request):
    latest_played = SomaLogLine.objects.select_related('track', 'track__artist').latest('play_timestamp')
    duration = latest_played.track.duration
    elapsed = datetime.datetime.now() - latest_played.play_timestamp
    remaining = duration - elapsed
    if remaining.total_seconds() < 2:
        return JsonResponse({})
    elif remaining.total_seconds() < 0:
        remaining = datetime.timedelta(seconds=0)
    response = {}
    if latest_played.track.artist:
        response['artist'] = latest_played.track.artist.name
    response['title'] = latest_played.track.title
    response['duration'] = duration.seconds
    response['elapsed'] = int(elapsed.total_seconds())
    return JsonResponse(response)
